﻿-- Author      : Grendizer@Doomhammer-EU
-- Create Date : 7/14/2011 5:52:14 PM

TestMode = 0;

BarItem = {}
BarItem.__index = BarItem
local frameCount = 0;


function BarItem.Create(playerName, stack, duration, fullDuration)
	local barItem = {}
	setmetatable(barItem, BarItem)
	barItem.playerName = playerName
	barItem.stack = stack;
	barItem.duration = duration;
	barItem.fullDuration = fullDuration;
	frameCount = frameCount + 1;
	barItem.frame = CreateFrame("StatusBar", "BHMBar"..frameCount, UIParent, "BHMBarTemplate");
	
	barItem.defaultWidth = barItem.frame:GetWidth()
	barItem.defaultHeight = barItem.frame:GetHeight()
	
	local fs = _G[barItem.frame:GetName().."PlayerName"]
	local font, height,flags = fs:GetFont();
	barItem.defaultFont = font
	barItem.defaultFontHeight = height
	barItem.defaultFlags = flags

	local _,classFileName = UnitClass(playerName);
	barItem.frame:SetMinMaxValues(0, 100) 
	local classColor = RAID_CLASS_COLORS[classFileName];
	barItem.frame:SetStatusBarColor(classColor["r"], classColor["g"], classColor["b"]);
	return barItem;
end

function BarItem:MoveBelow(otherBarItem)
	self.frame:SetPoint("TOPLEFT", otherBarItem, "BOTTOMLEFT", 0, 0);	
end

function BarItem:MoveTo(x, y)
	self.frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x, y);
end

function BarItem:Hide()
	self.frame:Hide()
end

function BarItem:Show()
	self.frame:Show()
end

function BarItem:SetValues(stack, duration, fullDuration)
	self.stack = stack
	self.duration = duration
	self.fullDuration = fullDuration
end

function BarItem:Update()
	_G[self.frame:GetName().."PlayerName"]:SetText(self.playerName .. self.stack);
	_G[self.frame:GetName().."Duration"]:SetText(self.duration .. "s");
	local min,max = self.frame:GetMinMaxValues();
	self.frame:SetValue(min + (max - min)*(self.duration/self.fullDuration));
	self.frame:Show()
end

function BarItem:Resize()
	self.frame:SetSize(self.defaultWidth + (BHMSettings["BarWidth"] or 50) - 50, self.defaultHeight * (BHMSettings["BarHeight"] or 50) / 50);
	local fontSize = self.defaultFontHeight * ((BHMSettings["BarFont"] or 50) + 30) / 80
	_G[self.frame:GetName().."PlayerName"]:SetFont(self.defaultFont, fontSize, self.defaultFlags);
	_G[self.frame:GetName().."Duration"]:SetFont(self.defaultFont, fontSize, self.defaultFlags);
	self.frame:SetAlpha((BHMSettings["BarOpacity"] or 50)/100);
end


BarCollection = {}
BarCollection.__index = BarCollection

AllBarCollections = {}

function BHMBarCollection_OnUpdate(self)
	if self.disabled == nil then
		self.barCollection:Update()
	end
end

function BarCollection.Create(name, spellId, x, y)
	local barCollection = {}
	setmetatable(barCollection, BarCollection);
	barCollection.barCount = 0
	barCollection.x = x
	barCollection.y = y
	barCollection.name = name
	
	barCollection.shownBars = {}
	barCollection.hiddenBars = {}
	
	barCollection.updateFrame = CreateFrame("Frame", "BHMBarCollection" .. name, UIParent);
	barCollection.updateFrame:SetPoint("BOTTOMLEFT", UIParent, "TOPLEFT", x, y);
	_,_,icon = GetSpellInfo(spellId)
	barCollection.updateFrame:SetSize(32,32);
	barCollection.updateFrame:SetMovable(1);
	barCollection.updateFrame:EnableMouse(1);
	local backdrop = {
		bgFile = icon,  
		tile = false,
		tileSize = 32,
	    edgeSize = 0,
	    insets = {
		    left = 0,
		    right = 0,
		    top = 0,
		    bottom = 0
		  }
		}
	barCollection.updateFrame:SetBackdrop(backdrop)
	barCollection.updateFrame:Show();
	barCollection.updateFrame.barCollection = barCollection
	barCollection.updateFrame:SetScript("OnUpdate", BHMBarCollection_OnUpdate);
	barCollection.updateFrame:SetScript("OnMouseDown", BHMBarCollection_OnMouseDown);
	barCollection.updateFrame:SetScript("OnMouseUp", BHMBarCollection_OnMouseUp);
	barCollection.updateFrame:SetScript("OnHide", BHMBarCollection_OnHide);
	barCollection.updateFrame:SetScript("OnEvent", BHMBarCollection_OnEvent);
	barCollection.updateFrame:RegisterEvent("PLAYER_REGEN_DISABLED")
	barCollection.updateFrame:RegisterEvent("PLAYER_REGEN_ENABLED")
	
	barCollection.lastUpdate = 0
	AllBarCollections[name] = barCollection
	
	return barCollection;
end

local function GetUnitCreatureId(uId)
	local guid = UnitGUID(uId)
	return (guid and (tonumber(guid:sub(7, 10), 16))) or 0
end


function BHMBarCollection_OnEvent(self, event, ...)
	if	event=="PLAYER_REGEN_ENABLED" then
		self.barCollection.iscombat = nil;
	elseif	event=="PLAYER_REGEN_DISABLED" then
		if GetUnitCreatureId("target") == 31146 or GetUnitCreatureId("boss1") == 53494 then
			self.barCollection.iscombat = 1;
		else
			self.barCollection.iscombat = nil;
		end
	end
	self.barCollection:Enable();
end

function BHMBarCollection_OnMouseDown(self, button)
  if button == "LeftButton" and not self.isMoving then
   self:StartMoving();
   self.isMoving = true;
  end
end

function BHMBarCollection_OnMouseUp(self, button)
  if button == "LeftButton" and self.isMoving then
   self:StopMovingOrSizing();
   self.isMoving = false;
  end
end

function BHMBarCollection_OnHide(self)
  if ( self.isMoving ) then
   self:StopMovingOrSizing();
   self.isMoving = false;
  end
end

function BarCollection:CheckPlayer(playerName)
	local name,_,_,stack,_,duration,expires = UnitAura(playerName, self.name)
	if stack == nil then
		stack = ""
	else
		if stack == 0 then
			BHMTooltip:SetUnitAura(playerName, name)
			local tt = BHMTooltipTextLeft2:GetText()
			stack = tonumber(string.match(tt or "", "%d+")) or 0
			if stack == 0 then
				stack = ""
			else
				stack = " (" .. stack .. "%)"
			end
		else
			stack = " (" .. stack .. ")"
		end
	end
	if duration == nil then
		duration = 0
	end
	local n = GetTime()
	if expires == nil then
		expires = n
	end
	self:ShowBar(playerName, stack, math.floor((expires - n)*10)/10, duration)
end

function BarCollection:Update()
	local now = GetTime()
	if (now - self.lastUpdate) < 0.1 then 
		return
	end
	if TestMode == 0 then
		self.lastUpdate = now
		local numraid = GetNumRaidMembers()
		if numraid == 0 then
			self:CheckPlayer(UnitName("player"))
			return
		end
		for i=1,numraid do
			local playerName = GetRaidRosterInfo(i)
			self:CheckPlayer(playerName)
		end
	else
		if self.testExpires == nil then
			self.testExpires = now + 30;
		elseif self.testExpires < now then
			self.testExpires = now + 30;
		end
		self:ShowBar(UnitName("player"), "", math.floor((self.testExpires - now) * 10)/10, 30)
	end
end

function BarCollection:ShowBar(playerName, stack, duration, fullDuration)
	if duration == 0 then
		local barItem = self.shownBars[playerName]
		if barItem == nil then
			return
		end
		self.shownBars[playerName] = nil
		self.hiddenBars[playerName] = barItem
		barItem:Hide()
	else
		local barItem = self.hiddenBars[playerName]
		if barItem ~= nil then
			self.hiddenBars[playerName] = nil
			barItem:SetValues(stack, duration, fullDuration)
			barItem:Resize()
			self.shownBars[playerName] = barItem
		else
			barItem = self.shownBars[playerName]
			if barItem == nil then
				barItem = BarItem.Create(playerName, stack, duration, fullDuration)
				self.shownBars[playerName] = barItem
				barItem:Resize()
			else
				barItem:SetValues(stack, duration, fullDuration)
			end
		end
		barItem:Update()
	end
	self:RestackBars()
end

function BarCollection:RestackBars()
	local previousBar = nil
	for i, barItem in pairs(self.shownBars) do
		if previousBar == nil then
			barItem:MoveBelow(self.updateFrame)
		else
			barItem:MoveBelow(previousBar.frame)
		end
		previousBar = barItem
	end
end

function BarCollection.ResizeAll()
	for name, barCollection in pairs(AllBarCollections) do
		barCollection:Resize()
	end
end

function BarCollection:Resize()
	for name, barItem in pairs(self.shownBars) do
		barItem:Resize()
	end
end

function BarCollection:Enable()
	if  not ((BHMSettings[self.name] ~= 0) and (self.iscombat == 1 or TestMode == 1)) then
		self.disabled = 1
		self.updateFrame:SetScript("OnUpdate", nil);
		self.updateFrame:Hide()
		for name, barItem in pairs(self.shownBars) do
			barItem:Hide()
			self.hiddenBars[barItem.playerName] = barItem;
		end
		self.shownBars = {}
	else
		self.disabled = nil
		self.updateFrame:SetScript("OnUpdate", BHMBarCollection_OnUpdate);
		self.updateFrame:Show()		
	end
end

function BarCollection.EnableCollection(name)
	local collection = AllBarCollections[name]
	if collection ~= nil then
		collection:Enable()
	end
end

function BarCollection.EnableAll()
	for name, collection in pairs(AllBarCollections) do
		collection:Enable()
	end
end

function Initialize()
	CreateFrame("GameTooltip", "BHMTooltip", nil, "GameTooltipTemplate")
	BHMTooltip:SetOwner(WorldFrame, "ANCHOR_NONE");
	
	for i, id in pairs(TrackedSpells) do
		BarCollection.Create(TrackedSpellNames[i], id, 200 + i * 200, -200):Enable();
	end
end


Initialize()

